"""
Grid class
"""

import numpy as np
from scipy import ndimage

class Grid:
    """
    Implementation of 2D grid of cells
    Includes boundary handling
    """

    def __init__(self, grid_height, grid_width):
        """
        Initializes grid to be empty, take height and width of grid as parameters
        Indexed by rows (left to right), then by columns (top to bottom)
        """
        #print('Grid initsialised with', grid_height, grid_width)
        self._grid_height = grid_height
        self._grid_width = grid_width
        self._cells = np.zeros((self._grid_height,self._grid_width))
        self.empty, self.full, self.barrier, self.corner = 0, 1, -1, -2
        self._geo_info = {'points': None, 'lines': None}
        self._compressed_grid = None
        self._bfs_search_data = None

    @property
    def cells(self):
        return self._cells

    @cells.setter
    def cells(self, val):
        self._cells = val

    @property
    def bfs_search_data(self):
        return self._bfs_search_data

    @bfs_search_data.setter
    def bfs_search_data(self, val):
        self._bfs_search_data = val

    def __str__(self):
        """
        Return multi-line string represenation for grid
        """
        ans = ""
        for row in range(self._grid_height):
            ans += str(self._cells[row])
            ans += "\n"
        return ans

    def make_empty_grid(self):
        '''
        Making a grid of all 0.
        :return:
        '''
        return np.zeros((self._grid_height,self._grid_width))

    def get_array(self):
        ''' Returning the numpy array '''
        return self._cells

    def provide_line_info(self, lines, points):
        ''' Line information to the grid.
            The geometric infomation is a dictionary
            {   line_dict: line_dict,
                point_dict: point_dict  }
        '''
        self._geo_info['lines'] = lines
        self._geo_info['points'] = points

    def get_grid_height(self):
        """
        Return the height of the grid for use in the GUI
        """
        return self._grid_height

    def get_grid_width(self):
        """
        Return the width of the grid for use in the GUI
        """
        return self._grid_width

    def get_matrix(self):
        """
        Return the complete matrix in numpy list form.
        """
        return self._cells

    def get_highest_number_in_grid(self):
        '''
        Retruns the highes number in the grid.
        :return:
        '''

        return np.amax(self._cells)

    def clear(self):
        """
        Clears grid to be empty
        """
        self._cells = np.zeros((self._grid_height,self._grid_width))

    def set_empty(self, row, col):
        """
        Set cell with index (row, col) to be empty
        """
        self._cells[row][col] = self.empty

    def set_full(self, row, col):
        """
        Set cell with index (row, col) to be full
        """
        self._cells[row][col] = self.full

    def set_value(self, row, col, value):
        """
        Set cell with index (row, col) to be a specified integer
        """
        self._cells[row][col] = value

    def set_barrier(self, row, col, line_number: int = None):
        """
        Set cell with index (row, col) to be full
        """

        if line_number is None:
            self._cells[row][col] = self.barrier
        else:
            self._cells[row][col] = line_number



    def set_number_to_cell(self, row, col, number):
        '''
        Setting an arbritary number to a cell.
        '''
        self._cells[row][col] = number

    def is_empty(self, row, col):
        """
        Checks whether cell with index (row, col) is empty
        """
        return self._cells[row][col] == self.empty

    def is_full(self, row, col):
        """
        Checks whether cell with index (row, col) is empty
        """
        return self._cells[row][col] == self.full

    def is_barrier(self, row, col):
        """
        Checks whether cell with index (row, col) is empty
        """
        return self._cells[row][col] == self.barrier

    def is_corner(self,point):
        '''
        Identifying corners.
        :param point:
        :return:
        '''
        return [self.get_value(item[0],item[1]) for item in
                self.eight_neighbors(point[0],point[1])].count(self.barrier) > 4

    def four_neighbors_extend_1(self, row, col):
        """
        Returns horiz/vert neighbors of cell (row, col)
        """
        ans = []
        if row > 0:
            ans.append((row - 2, col))
        if row < self._grid_height - 1:
            ans.append((row + 2, col))
        if col > 0:
            ans.append((row, col - 2))
        if col < self._grid_width - 1:
            ans.append((row, col + 2))
        return ans

    def four_neighbors(self, row, col):
        """
        Returns horiz/vert neighbors of cell (row, col)
        """
        ans = []
        if row > 0:
            ans.append((row - 1, col))
        if row < self._grid_height - 1:
            ans.append((row + 1, col))
        if col > 0:
            ans.append((row, col - 1))
        if col < self._grid_width - 1:
            ans.append((row, col + 1))
        return ans

    def eight_neighbors(self, row, col):
        """
        Returns horiz/vert neighbors of cell (row, col) as well as
        diagonal neighbors
        """
        ans = []
        if row > 0:
            ans.append((row - 1, col))
        if row < self._grid_height - 1:
            ans.append((row + 1, col))
        if col > 0:
            ans.append((row, col - 1))
        if col < self._grid_width - 1:
            ans.append((row, col + 1))
        if (row > 0) and (col > 0):
            ans.append((row - 1, col - 1))
        if (row > 0) and (col < self._grid_width - 1):
            ans.append((row - 1, col + 1))
        if (row < self._grid_height - 1) and (col > 0):
            ans.append((row + 1, col - 1))
        if (row < self._grid_height - 1) and (col < self._grid_width - 1):
            ans.append((row + 1, col + 1))
        return ans

    def get_index(self, point, cell_size):
        """
        Takes point in screen coordinates and returns index of
        containing cell
        """
        return (point[1] / cell_size, point[0] / cell_size)

    def get_value(self, row, col):
        # print('requested ROW COL: ', row, col)
        # print('CURRENT CELL LEN: ',len(self._cells), 'Shape is ',self._cells.shape)
        #print(self._cells[row])
        return self._cells[row][col]

    def get_points_along_line(self,start, end):
        """Bresenham's Line Algorithm
        Produces a list of tuples from start and end
            points1 = get_line((0, 0), (3, 4))
            points2 = get_line((3, 4), (0, 0))
            assert(set(points1) == set(points2))
            print points1
            [(0, 0), (1, 1), (1, 2), (2, 3), (3, 4)]
            [(3, 4), (2, 3), (1, 2), (1, 1), (0, 0)]
        """
        # Setup initial conditions
        x1 = int(start[0])
        y1 = int(start[1])
        x2 = int(end[0])
        y2 = int(end[1])

        dx = x2 - x1
        dy = y2 - y1

        # Determine how steep the line is
        is_steep = abs(dy) > abs(dx)

        # Rotate line
        if is_steep:
            x1, y1 = y1, x1
            x2, y2 = y2, x2

        # Swap start and end points if necessary and store swap state
        swapped = False
        if x1 > x2:
            x1, x2 = x2, x1
            y1, y2 = y2, y1
            swapped = True

        # Recalculate differentials
        dx = x2 - x1
        dy = y2 - y1

        # Calculate error
        error = int(dx / 2.0)
        ystep = 1 if y1 < y2 else -1

        # Iterate over bounding box generating points between start and end
        y = y1
        points = []
        for x in range(x1, x2 + 1):
            coord = (y, x) if is_steep else (x, y)
            points.append(coord)
            error -= abs(dy)
            if error < 0:
                y += ystep
                error += dx

        # Reverse the list if the coordinates were swapped
        if swapped:
            points.reverse()
        return points

    def get_mid_point(self, cell1, cell2):
        '''
        Get the point that is in the middle between two points.
        :param point1:
        :param point2:
        :return:
        '''
        idx = int(round(len(self.get_points_along_line(cell1,cell2))/2,0))
        return self.get_points_along_line(cell1,cell2)[idx]

    def get_adjacent_values(self,cell):
        '''
        Find the labels in the grid adjacent to the specified point.
        :param point:
        :return:
        '''
        # return tuple(set([int(self.get_value(neighbor[0], neighbor[1]))
        #                   for neighbor in self.four_neighbors_extend_1(cell[0], cell[1])]))
        return tuple(set([int(self.get_value(neighbor[0], neighbor[1]))
                          for neighbor in self.four_neighbors(cell[0], cell[1])]))

    def get_adjacent_values_duplicates(self,cell):
        '''
        Find the labels in the grid adjacent to the specified point.
        :param point:
        :return:
        '''

        return_tuple = tuple(list([int(self.get_value(neighbor[0], neighbor[1]))
                                   for neighbor in self.four_neighbors(cell[0], cell[1])]))
        # return_tuple = tuple(list([int(self.get_value(neighbor[0], neighbor[1]))
        #                            for neighbor in self.four_neighbors_extend_1(cell[0], cell[1])]))

        len(tuple(set(return_tuple)))

        if len(tuple(set(return_tuple))) > 1:
            return set(return_tuple)
        else:
            return (tuple(set(return_tuple))[0],tuple(set(return_tuple))[0])

    def get_highest_cell(self, value):
        '''
        Get the cell closes to (0,0) with a given value.
        :param value:
        :return:
        '''

        highest = (self.get_grid_height(),0)
        for row in range(self.get_grid_height()):
            for col in range(self.get_grid_width()):
                if self.get_value(row,col) == value and row < highest[0]:
                    highest = (row,col)
        return highest

    def get_lowest_cell(self,value):
        '''
        Get the cell closest to (height,0) with a given value.
        :param value:
        :return:
        '''

        lowest = (0,0)

        for row in range(self.get_grid_height()):
            for col in range(self.get_grid_width()):
                if self.get_value(row,col) == value and row > lowest[0]:
                    lowest = (row,col)
        return lowest

    def get_number_of_cells_with_value(self,value):
        '''
        Get the number of cells with a certain value.
        :param value:
        :return:
        '''

        counter = 0
        for row in range(self.get_grid_height()):
            for col in range(self.get_grid_width()):
                if self.get_value(row,col) == value:
                    counter += 1
        return counter

    def import_grid(self,grid):
        '''
        Import a grid to replace created grid. Converting to numpy array.
        :param grid:
        :return:
        '''
        if np.array(grid, dtype=object).shape[1] == 2:
            self._cells = self.rebuild_compressed(grid)
        else:
            self.cells = np.array(grid) # old large save files

    def export_grid(self):
        '''
        Converting from array to list of list. Exporting the grid.
        :return:
        '''
        return self.export_compressed_grid()

    def export_compressed_grid(self):
        '''
        Converting from array to list of list. Exporting the grid for saving.
        :return:
        '''
        save_list = list()

        # Compressing horizontally
        for row in self._cells:
            this_counter, this_number, save_row = 1, row[0], list()
            for col_idx in range(len(row)-1):
                last = col_idx == len(row)-2
                if row[col_idx] == row[col_idx+1] and not last:
                    this_counter += 1
                elif row[col_idx] != row[col_idx+1] and not last:
                    save_row.append([this_number, this_counter])
                    this_number = row[col_idx+1]
                    this_counter = 1
                elif last:
                    save_row.append([this_number, this_counter+1])
            save_list.append(save_row)

        # Compressing vertically
        this_counter, this_number, save_vertical = 0, save_list[0], list()
        for row_idx in range(len(save_list) - 1):
            last = row_idx == len(save_list) - 2
            if save_list[row_idx] == save_list[row_idx +1] and not last:
                this_counter += 1
            elif save_list[row_idx] != save_list[row_idx +1] and not last:
                save_vertical.append([this_number, this_counter])
                this_number = save_list[row_idx +1]
                this_counter = 1
            elif last:
                save_vertical.append([this_number, this_counter])
                if save_list[row_idx+1] != save_list[row_idx]:
                    save_vertical.append([save_list[row_idx+1], 1])
                if save_vertical[0][-1] != self.cells.shape[0]:
                    save_vertical[0][-1] += 1
                # assert this_counter == self.cells.shape[0], 'Error. Shape of compressed grid is not equal to input grid. ' \
                #                                             'The counter has skipped a step. The saved is '\
                #                                             +str(save_vertical) + ' and the shape is ' + \
                #                                             str(self.cells.shape)


        return save_vertical

    def rebuild_compressed(self, compressed_grid = None):
        '''
        Rebuilding a compressed grid made by 'export_compressed_grid(self)'
        :return:
        '''
        compressed_grid = compressed_grid if compressed_grid is not None else self._compressed_grid
        vertical_expansion_list = []

        # Expand vertically
        for row_count, row in enumerate(compressed_grid):
            values = row[0]
            value_count = row[1]
            for dummy_i in range(value_count):
                vertical_expansion_list.append(values)

        # Expand horisontally
        expanded_list = [list() for dummy_i in range(len(vertical_expansion_list))]

        for row_count, row in enumerate(vertical_expansion_list):
            for values in row:
                value = values[0]
                value_count = values[1]
                for dummy_i in range(value_count):
                    expanded_list[row_count].append(value)
        #print('Shape of rebuilt grid is',np.array(expanded_list).shape)
        return np.array(expanded_list)

    def get_center_of_matrix(self, height_limit: float = None, scale: float = 10):
        '''

        '''
        import copy
        calc_grid = copy.deepcopy(self.get_array())
        all_compartments = np.unique(calc_grid)
        calc_grid[calc_grid == 1] = 0

        #set all values to a value
        for val in all_compartments:
            if val != 0:
                calc_grid[calc_grid == val] = 1

        if height_limit != None:
            calc_grid = calc_grid[int(self._grid_height-height_limit*scale):, :]

        center_of_mass = ndimage.measurements.center_of_mass(calc_grid)

        center_of_mass = ((calc_grid.shape[0]-center_of_mass[0])/scale, center_of_mass[1]/scale)
        return center_of_mass

if __name__ ==  '__main__':
    import example_data as ex
    import grid_window as grd
    from matplotlib import pyplot as plt
    lines = ex.line_dict
    points = ex.point_dict

    canvas_dim = [1000,720]
    canvas_origo = (50,670)
    empty_grid = np.zeros((720, 1000))
    my_grid = grd.CreateGridWindow(ex.get_grid_no_inp(empty_grid=False), canvas_dim, {}, canvas_origo)
    print('before search', my_grid.grid.get_array().shape)
    search_return = my_grid.search_bfs(animate = True)
    print('after search', my_grid.grid.get_array().shape)
    my_grid.grid.get_center_of_matrix(height_limit=5)





# This is where the optimization is done.
import tkinter as tk

from _tkinter import TclError
import os
try:
    import any_files.example_data as test
except ModuleNotFoundError:
    import ANYstructure.any_files.example_data as test


class CreateStressesWindow():
    '''
    This class initiates the GUI used to define stresses for the selected structure.
    '''
    def __init__(self, master, app=None):
        super(CreateStressesWindow, self).__init__()
        if __name__ == '__main__':
            self._initial_structure_obj = test.get_structure_object()
            self.default_stresses = test.get_default_stresses()
            image_dir = os.path.dirname(__file__) + '\\images\\'
        else:
            self.app = app
            try:
                self._initial_structure_obj = app._line_to_struc[app._active_line][0]
            except KeyError:
                self._initial_structure_obj = None
            self.default_stresses = app._default_stresses
            image_dir = app._root_dir + '\\images\\'

        self._frame = master
        self._frame.wm_title("Specify strucutre - returned to input field in main window")
        self._frame.geometry('1500x900')
        self._frame.grab_set()

        self._opt_runned = False
        self._opt_resutls = ()

        self._draw_scale = 500
        self._canvas_dim = (500, 450)

        tk.Label(self._frame, text='-- Global stresses and fixation parameter in plate/stiffener --',
                 font='Verdana 15 bold').place(x=10, y=10)

        ent_w = 10
        # stresses in plate and stiffener

        self._new_structure_type = tk.StringVar()
        self._new_trans_stress_high = tk.DoubleVar()
        self._new_trans_stress_low = tk.DoubleVar()
        self._new_axial_stress_1 = tk.DoubleVar()
        self._new_axial_stress_2 = tk.DoubleVar()
        self._new_shear_stress = tk.DoubleVar()
        self._new_km1 = tk.DoubleVar()
        self._new_km2 = tk.DoubleVar()
        self._new_km3 = tk.DoubleVar()
        self._new_kpp = tk.DoubleVar()
        self._new_kps = tk.DoubleVar()
        self._new_max_pressure_side = tk.StringVar()

        self._ent_structure_type = tk.OptionMenu(self._frame,self._new_structure_type,command=self.change_option_menu,
                                                *self.default_stresses.keys())
        self._ent_trans_stress_high = tk.Entry(self._frame, textvariable=self._new_trans_stress_high, width=ent_w)
        self._ent_trans_stress_low = tk.Entry(self._frame, textvariable=self._new_trans_stress_low, width=ent_w)
        self._ent_axial_stress_1 = tk.Entry(self._frame, textvariable=self._new_axial_stress_1, width=ent_w)
        self._ent_axial_stress_2 = tk.Entry(self._frame, textvariable=self._new_axial_stress_2, width=ent_w)
        self._ent_shear_stress = tk.Entry(self._frame, textvariable=self._new_shear_stress, width=ent_w)
        self._ent_km1 = tk.Entry(self._frame, textvariable=self._new_km1, width=ent_w)
        self._ent_km2 = tk.Entry(self._frame, textvariable=self._new_km2, width=ent_w)
        self._ent_km3 = tk.Entry(self._frame, textvariable=self._new_km3, width=ent_w)
        self._ent_kpp = tk.Entry(self._frame, textvariable=self._new_kpp, width=ent_w)
        self._ent_kps = tk.Entry(self._frame, textvariable=self._new_kps, width=ent_w)
        self._ent_pressure_side = tk.OptionMenu(self._frame,self._new_max_pressure_side,*('p','s'))

        start_x,start_y,dx,dy = 20,100,100,35

        ###
        # tk.Label(self._frame, text='Input stresses and parameters:', font='Verdana 12 bold',fg='red') \
        #     .place(x=start_x , y=start_y + 9 * dy)

        tk.Label(self._frame, text='Select strucutre type:', font='Verdana 9',fg='red') \
            .place(x=start_x , y=start_y + 10 * dy)

        tk.Label(self._frame, text='Sigma,y1_Sd - large transversal stress', font='Verdana 9') \
            .place(x=start_x , y=start_y + 11 * dy)
        tk.Label(self._frame, text='[MPa]', font='Verdana 9 bold') \
            .place(x=start_x + dx * 4, y=start_y + 11 * dy)

        tk.Label(self._frame, text='Sigma,y2_Sd - small transversal stress', font='Verdana 9') \
            .place(x=start_x, y=start_y + 12* dy)
        tk.Label(self._frame, text='[MPa]', font='Verdana 9 bold') \
            .place(x=start_x + dx * 4, y=start_y + 12 * dy)

        tk.Label(self._frame, text='Sigma,x_Sd - axial stress 1', font='Verdana 9') \
            .place(x=start_x, y=start_y + 13 * dy)

        tk.Label(self._frame, text='[MPa]', font='Verdana 9 bold') \
            .place(x=start_x + dx * 4, y=start_y + 13 * dy)

        tk.Label(self._frame, text='Sigma,x_Sd - axial stress 2', font='Verdana 9') \
            .place(x=start_x, y=start_y + 14 * dy)

        tk.Label(self._frame, text='[MPa]', font='Verdana 9 bold') \
            .place(x=start_x + dx * 4, y=start_y + 14 * dy)

        tk.Label(self._frame, text='Tau,xy - shear stress', font='Verdana 9') \
            .place(x=start_x, y=start_y + 15 * dy)
        tk.Label(self._frame, text='[MPa]', font='Verdana 9 bold') \
            .place(x=start_x + dx * 4, y=start_y + 15 * dy)

        tk.Label(self._frame, text='km1, bending moment factor', font='Verdana 9') \
            .place(x=start_x, y=start_y + 16* dy)

        tk.Label(self._frame, text='km2, bending moment factor', font='Verdana 9') \
            .place(x=start_x, y=start_y + 17 * dy)

        tk.Label(self._frame, text='km3, bending moment factor', font='Verdana 9') \
            .place(x=start_x, y=start_y + 18 * dy)

        tk.Label(self._frame, text='kpp, fixation parameter plate', font='Verdana 9') \
            .place(x=start_x, y=start_y + 19 * dy)

        tk.Label(self._frame, text='kps, fixation parameter stiffener', font='Verdana 9') \
            .place(x=start_x, y=start_y + 20 * dy)

        tk.Label(self._frame, text='Max pressure side (plate of stiffener)', font='Verdana 9 bold') \
            .place(x=start_x+5*dx, y=start_y + 8 * dy)

        self._ent_structure_type.place(x=start_x + dx * 3, y=start_y + 10 * dy)
        self._ent_trans_stress_high.place(x=start_x + dx * 3, y=start_y + 11 * dy)
        self._ent_trans_stress_low.place(x=start_x + dx * 3, y=start_y + 12 * dy)
        self._ent_axial_stress_1.place(x=start_x + dx * 3, y=start_y + 13 * dy)
        self._ent_axial_stress_2.place(x=start_x + dx * 3, y=start_y + 14 * dy)
        self._ent_shear_stress.place(x=start_x + dx * 3, y=start_y + 15 * dy)
        self._ent_km1.place(x=start_x + dx * 3, y=start_y + 16 * dy)
        self._ent_km2.place(x=start_x + dx * 3, y=start_y + 17 * dy)
        self._ent_km3.place(x=start_x + dx * 3, y=start_y + 18 * dy)
        self._ent_kpp.place(x=start_x + dx * 3, y=start_y + 19 * dy)
        self._ent_kps.place(x=start_x + dx * 3, y=start_y + 20 * dy)
        self._ent_pressure_side.place(x=start_x+8*dx, y=start_y + 8 * dy)

        # setting default values
        init_dim = 0.05
        init_thk = 0.002

        if self._initial_structure_obj != None:
            self._new_trans_stress_high.set(self._initial_structure_obj.Plate.get_sigma_y1())
            self._new_trans_stress_low.set(self._initial_structure_obj.Plate.get_sigma_y2())
            self._new_axial_stress_1.set(self._initial_structure_obj.Plate.get_sigma_x1())
            self._new_axial_stress_2.set(self._initial_structure_obj.Plate.get_sigma_x2())
            self._new_shear_stress.set(self._initial_structure_obj.Plate.get_tau_xy())
            self._new_km1.set(self._initial_structure_obj.Plate.get_km1())
            self._new_km2.set(self._initial_structure_obj.Plate.get_km2())
            self._new_km3.set(self._initial_structure_obj.Plate.get_km3())
            self._new_kpp.set(self._initial_structure_obj.Plate.get_kpp())
            self._new_kps.set(self._initial_structure_obj.Plate.get_kps())
            self._new_structure_type.set(self._initial_structure_obj.Plate.get_structure_type())

        else:
            self._new_structure_type.set('GENERAL_INTERNAL_WT')
            self._new_trans_stress_high.set(self.default_stresses[self._new_structure_type.get()][0])
            self._new_trans_stress_low.set(self.default_stresses[self._new_structure_type.get()][1])
            self._new_axial_stress_1.set(self.default_stresses[self._new_structure_type.get()][2])
            self._new_axial_stress_1.set(self.default_stresses[self._new_structure_type.get()][3])
            self._new_shear_stress.set(self.default_stresses[self._new_structure_type.get()][4])
            self._new_km1.set(12)
            self._new_km2.set(24)
            self._new_km3.set(12)
            self._new_kpp.set(1)
            self._new_kps.set(1)
            self._new_max_pressure_side.set('p')
        try:
            img_file_name = 'img_transverse_stress.gif'
            if os.path.isfile('images/' + img_file_name):
                file_path ='images/' + img_file_name
            else:
                file_path = app._root_dir + '/images/' + img_file_name
            photo_transverse = tk.PhotoImage(file=file_path)
            label_trans = tk.Label(self._frame, image=photo_transverse)
            label_trans.image = photo_transverse  # keep a reference!
            label_trans.place(x=start_x, y=60)
        except TclError:
            pass

        try:
            img_file_name = "img_axial_stresses.gif"
            if os.path.isfile('images/' + img_file_name):
                file_path ='images/' + img_file_name
            else:
                file_path = app._root_dir + '/images/' + img_file_name
            photo_axial = tk.PhotoImage(file=file_path)
            label_axial = tk.Label(self._frame, image=photo_axial)
            label_axial.image = photo_axial  # keep a reference!
            label_axial.place(x=start_x+5*dx, y=60)
        except TclError:
            pass

        try:
            img_file_name = 'img_fixation_parameters.gif'
            if os.path.isfile('images/' + img_file_name):
                file_path ='images/' + img_file_name
            else:
                file_path = app._root_dir + '/images/' + img_file_name
            photo = tk.PhotoImage(file=file_path)
            label_fix = tk.Label(self._frame, image=photo)
            label_fix.image = photo  # keep a reference!
            label_fix.place(x=start_x+9.5*dx, y=60)
        except TclError:
            pass

        tk.Label(self._frame,text='The stresses are global values and is estimated '
                                 '\nby user.\n'
                                 'Alterntively read out stresses from FE-model.\n'
                                 'Suggestions for input:\n'
                                 'Transverse stresses (Sigma,y_Sd is calculated):\n'
                                 '   - conservative - about 100 MPa \n'
                                 '   - non-conservative - about 60 MPa\n'
                                 'Axial stresses: \n'
                                 '   - about 60 MPa\n'
                                 '   - non-conservative - about 40 MPa\n'
                                 'Shear stresses: \n'
                                 '   - about 20 MPa\n'
                                 '   - non-conservative - about 1 MPa', justify=tk.LEFT,
                 font = 'Verdana 10', fg = 'blue',bg='white')\
            .place(x=start_x+dx*4.5,y=start_y+dy*11)

        self._close_and_save = tk.Button(self._frame, text='Return and set stresses and fixation parameter',
                                        command=self.save_and_close, bg='green', font='Verdana 10', fg='yellow')
        self._close_and_save.place(x=start_x + dx * 4.5, y=start_y + dy * 19)

    def change_option_menu(self,event):
        '''
        Action when changing the structure type
        :return:
        '''
        self._new_trans_stress_high.set(self.default_stresses[self._new_structure_type.get()][0])
        self._new_trans_stress_low.set(self.default_stresses[self._new_structure_type.get()][1])
        self._new_axial_stress_1.set(self.default_stresses[self._new_structure_type.get()][2])
        self._new_axial_stress_1.set(self.default_stresses[self._new_structure_type.get()][3])
        self._new_shear_stress.set(self.default_stresses[self._new_structure_type.get()][4])

    def save_and_close(self):
        '''
        Save and close
        :return:
        '''
        if __name__ == '__main__':
            self._frame.destroy()
            return

        self.app.on_close_stresses_window([self._new_trans_stress_high.get(),
                                           self._new_trans_stress_low.get(),
                                           self._new_axial_stress_1.get(),
                                           self._new_axial_stress_2.get(),
                                           self._new_shear_stress.get(),
                                           self._new_km1.get(),
                                           self._new_km2.get(),
                                           self._new_km3.get(),
                                           self._new_kpp.get(),
                                           self._new_kps.get(),
                                           self._new_structure_type.get(),
                                           self._new_max_pressure_side.get()])
        self._frame.destroy()


if __name__ == '__main__':
    root = tk.Tk()
    my_app = CreateStressesWindow(root,app=None)
    root.mainloop()


# Audun

import tkinter as tk
import os
from _tkinter import TclError

class CreateLoadFactorWindow:
    '''
    self._load_factors_dict = {'dnva':[1.3,1.2,0.7], 'dnvb':[1,1,1.3], 'tanktest':[1,1,0]} # DNV  loads factors
    '''

    def __init__(self,master, app=None):
        super(CreateLoadFactorWindow, self).__init__()
        self._frame = master
        self._frame.wm_title("Load factor modifications here.")
        self._frame.geometry('800x800')
        self._frame.grab_set()
        self._app = app
        if __name__ == '__main__':
            self._load_factors_dict = {'dnva': [1.3, 1.2, 0.7], 'dnvb': [1, 1, 1.3],
                                       'tanktest': [1, 1, 0]}  # DNV  loads factors


        else:
            self._load_factors_dict = app._load_factors_dict

        self.new_conda_lff1 = tk.DoubleVar()
        self.new_conda_lff2 = tk.DoubleVar()
        self.new_conda_lff3 = tk.DoubleVar()
        self.new_condb_lff1 = tk.DoubleVar()
        self.new_condb_lff2 = tk.DoubleVar()
        self.new_condb_lff3 = tk.DoubleVar()
        self.new_condtt_lff1 = tk.DoubleVar()
        self.new_condtt_lff2 = tk.DoubleVar()
        self.new_condtt_lff3 = tk.DoubleVar()

        self.new_change_default = tk.BooleanVar()
        self.new_change_existing = tk.BooleanVar()

        self.new_conda_lff1.set(self._load_factors_dict['dnva'][0])
        self.new_conda_lff2.set(self._load_factors_dict['dnva'][1])
        self.new_conda_lff3.set(self._load_factors_dict['dnva'][2])
        self.new_condb_lff1.set(self._load_factors_dict['dnvb'][0])
        self.new_condb_lff2.set(self._load_factors_dict['dnvb'][1])
        self.new_condb_lff3.set(self._load_factors_dict['dnvb'][2])
        self.new_condtt_lff1.set(self._load_factors_dict['tanktest'][0])
        self.new_condtt_lff2.set(self._load_factors_dict['tanktest'][1])
        self.new_condtt_lff3.set(self._load_factors_dict['tanktest'][2])

        ent_w = 20
        tk.Label(self._frame, text='Static and dynamic load factors is specified here', font='Verdana 15 bold')\
            .grid(row = 1, column = 1, sticky = tk.W)
        tk.Label(self._frame, text='Note that DNV is used as reference, '
                                   'but the load factors can be any other rule set such as ISO.', font='Verdana 8 bold')\
            .grid(row = 2, column = 1, sticky = tk.W)

        tk.Label(self._frame, text=' ', font='Verdana 8 bold')\
            .grid(row = 3, column = 1, sticky = tk.W)

        tk.Label(self._frame, text='Condition a) - Static load factor "unknown loads"', font='Verdana 8 bold')\
            .grid(row = 4, column = 1, sticky = tk.W)
        tk.Label(self._frame, text='Condition a) - Static load factor well defined loads', font='Verdana 8 bold')\
            .grid(row = 5, column = 1, sticky = tk.W)
        tk.Label(self._frame, text='Condition a) - Dynamic load factor', font='Verdana 8 bold')\
            .grid(row = 6, column = 1, sticky = tk.W)
        self.ent_conda_lf1 = tk.Entry(self._frame, textvariable=self.new_conda_lff1, width=ent_w)
        self.ent_conda_lf2 = tk.Entry(self._frame, textvariable=self.new_conda_lff2, width=ent_w)
        self.ent_conda_lf3 = tk.Entry(self._frame, textvariable=self.new_conda_lff3, width=ent_w)
        self.ent_conda_lf1.grid(row=4, column=2)
        self.ent_conda_lf2.grid(row=5, column=2)
        self.ent_conda_lf3.grid(row=6, column=2)
        


        tk.Label(self._frame, text=' ', font='Verdana 8 bold')\
            .grid(row = 7, column = 1, sticky = tk.W)

        tk.Label(self._frame, text='Condition b) - Static load factor "unknown loads"', font='Verdana 8 bold')\
            .grid(row = 8, column = 1, sticky = tk.W)
        tk.Label(self._frame, text='Condition b) - Static load factor well defined loads', font='Verdana 8 bold')\
            .grid(row = 9, column = 1, sticky = tk.W)
        tk.Label(self._frame, text='Condition b) - Dynamic load factor', font='Verdana 8 bold')\
            .grid(row = 10, column = 1, sticky = tk.W)
        self.ent_condb_lf1 = tk.Entry(self._frame, textvariable=self.new_condb_lff1, width=ent_w)
        self.ent_condb_lf2 = tk.Entry(self._frame, textvariable=self.new_condb_lff2, width=ent_w)
        self.ent_condb_lf3 = tk.Entry(self._frame, textvariable=self.new_condb_lff3, width=ent_w)
        self.ent_condb_lf1.grid(row=8, column=2)
        self.ent_condb_lf2.grid(row=9, column=2)
        self.ent_condb_lf3.grid(row=10, column=2)

        tk.Label(self._frame, text=' ', font='Verdana 8 bold')\
            .grid(row = 11, column = 1, sticky = tk.W)

        tk.Label(self._frame, text='Tank test) - Static load factor "unknown loads"', font='Verdana 8 bold')\
            .grid(row = 12, column = 1, sticky = tk.W)
        tk.Label(self._frame, text='Tank test)  - Static load factor well defined loads', font='Verdana 8 bold')\
            .grid(row = 13, column = 1, sticky = tk.W)
        tk.Label(self._frame, text='Tank test)  - Dynamic load factor', font='Verdana 8 bold')\
            .grid(row = 14, column = 1, sticky = tk.W)
        self.ent_condtt_lf1 = tk.Entry(self._frame, textvariable=self.new_condtt_lff1, width=ent_w)
        self.ent_condtt_lf2 = tk.Entry(self._frame, textvariable=self.new_condtt_lff2, width=ent_w)
        self.ent_condtt_lf3 = tk.Entry(self._frame, textvariable=self.new_condtt_lff3, width=ent_w)
        self.ent_condtt_lf1.grid(row=12, column=2)
        self.ent_condtt_lf2.grid(row=13, column=2)
        self.ent_condtt_lf3.grid(row=14, column=2)

        tk.Label(self._frame, text=' ', font='Verdana 8 bold')\
            .grid(row = 15, column = 1, sticky = tk.W)
        # tk.Label(self._frame, text='Change all current load factors', font='Verdana 8 bold')\
        #     .grid(row = 16, column = 1, sticky = tk.W)
        # tk.Checkbutton(self._frame, variable=self.new_change_existing)\
        #     .grid(row=17, column=1, sticky = tk.W)
        # tk.Label(self._frame, text='Change default load factors', font='Verdana 8 bold')\
        #     .grid(row = 18, column = 1, sticky = tk.W)
        # tk.Checkbutton(self._frame, variable=self.new_change_default)\
        #     .grid(row=19, column=1, sticky=tk.W)
        #
        tk.Label(self._frame, text=' ', font='Verdana 8 bold')\
            .grid(row = 16, column = 1, sticky = tk.W)

        destroy_and_return = tk.Button(self._frame, text='Return specified load factors and change existing',
                                        command=self.return_load_factors, bg='green', font='Verdana 12', fg='yellow')
        destroy_and_return.grid(row = 17, column = 1)

        tk.Label(self._frame, text=' ', font='Verdana 8 bold')\
            .grid(row = 18, column = 1)

        try:
            img_file_name = 'img_dnv_load_combinations.gif'
            if os.path.isfile('images/' + img_file_name):
                file_path ='images/' + img_file_name
            else:
                file_path = app._root_dir + '/images/' + img_file_name
            photo_transverse = tk.PhotoImage(file=file_path)
            label_trans = tk.Label(self._frame, image=photo_transverse)
            label_trans.image = photo_transverse  # keep a reference!
            label_trans.grid(row = 19, column = 1, columnspan = 2)
        except TclError:
            pass

    def return_load_factors(self):
        '''
        self._load_factors_dict = {'dnva':[1.3,1.2,0.7], 'dnvb':[1,1,1.3], 'tanktest':[1,1,0]} # DNV  loads factors
        :return:
        '''
        self._load_factors_dict['dnva'] = [self.new_conda_lff1.get(), self.new_conda_lff2.get(),
                                           self.new_conda_lff3.get()]
        self._load_factors_dict['dnvb'] = [self.new_condb_lff1.get(), self.new_condb_lff2.get(),
                                           self.new_condb_lff3.get()]
        self._load_factors_dict['tanktest'] = [self.new_condtt_lff1.get(), self.new_condtt_lff2.get(),
                                           self.new_condtt_lff3.get()]
        if __name__ == '__main__':
            self._frame.destroy()
            print({'returned lf dict': self._load_factors_dict,
                                               'change exisiting': self.new_change_existing.get(),
                                               'change default': self.new_change_default.get()})
            return

        self._app.on_close_load_factor_window({'returned lf dict': self._load_factors_dict,
                                               'change exisiting': self.new_change_existing.get(),
                                               'change default': self.new_change_default.get()})
        self._frame.destroy()
if __name__ == '__main__':
    root = tk.Tk()
    my_app = CreateLoadFactorWindow(root,app=None)
    root.mainloop()